#! /usr/bin/env r
## Author: Arne Köhn <arne@chark.eu>
## License: Apache v2

## Usage: Run it, wait, obtain the results in the main table of our
## paper "An Empirical Analysis of the Correlation of Syntax and
## Prosody" (Köhn, Baumann, Dörfler 2018)

library(lme4)

cnames <- c("article", "word", "pitch", "power", "tact", "tpred",
            "deplabel", "slength", "wpos", "plength", "ppresent")

datasets <- c(
    "SUBJ-OBJA",
    "DET_(SUBJ-OBJA)",
    "ATTR_(SUBJ-OBJA)",
    "S-NEB",
    "S-REL",
    "S-AUX",
    "AUX_(S-NEB)",
    "AUX_(S-REL)",
    "AUX_(S-AUX)"
)

resrows <- c("pitch_sig", "pitch", "power_sig", "power", "duration_sig", "duration", "pause_sig", "pause")

# construct output, name rows and columns
dframe <- data.frame(matrix(ncol=NROW(datasets), nrow=NROW(resrows)), row.names = resrows)
colnames(dframe) <- datasets

sigfunc  <- function (x) {
    sig <- if (x < 0.001) "***" else if (x < 0.01) "**" else if (x < 0.05) "*" else if (x < 0.1) "." else "-"
    paste(sig, round(x,2), sep=",")
}

for (dset in datasets) {
    dsource <- paste("matrix-", dset, ".txt", sep="")
    print(paste("processing", dsource))
    A <- read.table(dsource, sep="\t")
    colnames(A) <- cnames

    ## normalize numerical predictors
    Ascaled <- A
    toscale <- c("tpred", "slength", "wpos")
    Ascaled[toscale] <- lapply(A[toscale], scale)

    ## fit all models and store them in an environment (env == hash map for R users...)
    lmermodels <- new.env()
    lmermodels[[paste("duration","base")]] <- lmer(tact ~ tpred*wpos*slength + (tpred|article), Ascaled)
    lmermodels[[paste("duration","ext")]] <- update(lmermodels[[paste("duration","base")]], .~. + deplabel*tpred)
    lmermodels[[paste("pitch","base")]] <- lmer(pitch ~ tpred*wpos*slength + (tpred|article) , Ascaled)
    lmermodels[[paste("pitch","ext")]] <- update(lmermodels[[paste("pitch","base")]], .~. + deplabel)
    lmermodels[[paste("power","base")]] <- lmer(power ~ 1 + tpred*wpos*slength + (tpred|article) , Ascaled)
    lmermodels[[paste("power","ext")]] <- update(lmermodels[[paste("power","base")]], .~. + deplabel)
    lmermodels[[paste("pause","base")]] <- lmer(plength ~ 1 + tpred*wpos*slength + (tpred|article) , Ascaled)
    lmermodels[[paste("pause","ext")]] <- update(lmermodels[[paste("pause","base")]], .~. + deplabel)

    aspects <- resrows[c(2,4,6,8)]
    for (aspect in aspects) {
        ## get base and extended models
        base <- lmermodels[[paste(aspect, "base")]]
        ext <- lmermodels[[paste(aspect, "ext")]]
        ## perform a likelihood ratio test comparing the fit of these models
        myanova <- anova(base, ext)
        ## fit a linear models on the base with dependency label as
        ## binary feature this tells us the magnitude of the fit.
        ## Can't use the coefficient of the ext model as that one
        ## could be too large due to correlation with other features.
        base.residuals <- resid(base)
        mylm <- lm(base.residuals ~ A$deplabel)
        ## finally, store the results
        dframe[aspect,dset] <- round(coef(mylm)[2], 4)
        dframe[paste(aspect, "_sig", sep=""), dset] <- sigfunc(myanova["ext","Pr(>Chisq)"])
    }
}
print.data.frame(dframe)
